@@ -2,7 +2,8 @@ import argparse
 import json
 import logging
 
-from borgmatic.borg import environment, feature, flags, rinfo
+import borgmatic.config.paths
+from borgmatic.borg import environment, feature, flags, repo_info
 from borgmatic.execute import DO_NOT_CAPTURE, execute_command
 
 logger = logging.getLogger(__name__)
@@ -50,10 +51,10 @@ def make_archive_filter_flags(local_borg_version, config, checks, check_argument
     return ()
 
 
-def make_check_flags(checks, archive_filter_flags):
+def make_check_name_flags(checks, archive_filter_flags):
     '''
-    Given a parsed checks set and a sequence of flags to filter archives,
-    transform the checks into tuple of command-line check flags.
+    Given parsed checks set and a sequence of flags to filter archives, transform the checks into
+    tuple of command-line check flags.
 
     For example, given parsed checks of:
 
@@ -94,7 +95,7 @@ def get_repository_id(
     '''
     try:
         return json.loads(
-            rinfo.display_repository_info(
+            repo_info.display_repository_info(
                 repository_path,
                 config,
                 local_borg_version,
@@ -134,10 +135,30 @@ def check_archives(
     if logger.isEnabledFor(logging.DEBUG):
         verbosity_flags = ('--debug', '--show-rc')
 
+    try:
+        repository_check_config = next(
+            check for check in config.get('checks', ()) if check.get('name') == 'repository'
+        )
+    except StopIteration:
+        repository_check_config = {}
+
+    if check_arguments.max_duration and 'archives' in checks:
+        raise ValueError('The archives check cannot run when the --max-duration flag is used')
+    if repository_check_config.get('max_duration') and 'archives' in checks:
+        raise ValueError(
+            'The archives check cannot run when the repository check has the max_duration option set'
+        )
+
+    max_duration = check_arguments.max_duration or repository_check_config.get('max_duration')
+
+    borg_environment = environment.make_environment(config)
+    borg_exit_codes = config.get('borg_exit_codes')
+
     full_command = (
         (local_path, 'check')
         + (('--repair',) if check_arguments.repair else ())
-        + make_check_flags(checks, archive_filter_flags)
+        + (('--max-duration', str(max_duration)) if max_duration else ())
+        + make_check_name_flags(checks, archive_filter_flags)
         + (('--remote-path', remote_path) if remote_path else ())
         + (('--log-json',) if global_arguments.log_json else ())
         + (('--lock-wait', str(lock_wait)) if lock_wait else ())
@@ -147,8 +168,7 @@ def check_archives(
         + flags.make_repository_flags(repository_path, local_borg_version)
     )
 
-    borg_environment = environment.make_environment(config)
-    borg_exit_codes = config.get('borg_exit_codes')
+    working_directory = borgmatic.config.paths.get_working_directory(config)
 
     # The Borg repair option triggers an interactive prompt, which won't work when output is
     # captured. And progress messes with the terminal directly.
@@ -157,6 +177,7 @@ def check_archives(
             full_command,
             output_file=DO_NOT_CAPTURE,
             extra_environment=borg_environment,
+            working_directory=working_directory,
             borg_local_path=local_path,
             borg_exit_codes=borg_exit_codes,
         )
@@ -164,6 +185,7 @@ def check_archives(
         execute_command(
             full_command,
             extra_environment=borg_environment,
+            working_directory=working_directory,
             borg_local_path=local_path,
             borg_exit_codes=borg_exit_codes,
         )
