@@ -5,9 +5,11 @@ import os
 import pathlib
 import stat
 import tempfile
+import textwrap
 
+import borgmatic.config.paths
 import borgmatic.logger
-from borgmatic.borg import environment, feature, flags, state
+from borgmatic.borg import environment, feature, flags
 from borgmatic.execute import (
     DO_NOT_CAPTURE,
     execute_command,
@@ -18,26 +20,28 @@ from borgmatic.execute import (
 logger = logging.getLogger(__name__)
 
 
-def expand_directory(directory):
+def expand_directory(directory, working_directory):
     '''
     Given a directory path, expand any tilde (representing a user's home directory) and any globs
     therein. Return a list of one or more resulting paths.
     '''
-    expanded_directory = os.path.expanduser(directory)
+    expanded_directory = os.path.join(working_directory or '', os.path.expanduser(directory))
 
     return glob.glob(expanded_directory) or [expanded_directory]
 
 
-def expand_directories(directories):
+def expand_directories(directories, working_directory=None):
     '''
-    Given a sequence of directory paths, expand tildes and globs in each one. Return all the
-    resulting directories as a single flattened tuple.
+    Given a sequence of directory paths and an optional working directory, expand tildes and globs
+    in each one. Return all the resulting directories as a single flattened tuple.
     '''
     if directories is None:
         return ()
 
     return tuple(
-        itertools.chain.from_iterable(expand_directory(directory) for directory in directories)
+        itertools.chain.from_iterable(
+            expand_directory(directory, working_directory) for directory in directories
+        )
     )
 
 
@@ -52,17 +56,19 @@ def expand_home_directories(directories):
     return tuple(os.path.expanduser(directory) for directory in directories)
 
 
-def map_directories_to_devices(directories):
+def map_directories_to_devices(directories, working_directory=None):
     '''
-    Given a sequence of directories, return a map from directory to an identifier for the device on
-    which that directory resides or None if the path doesn't exist.
+    Given a sequence of directories and an optional working directory, return a map from directory
+    to an identifier for the device on which that directory resides or None if the path doesn't
+    exist.
 
     This is handy for determining whether two different directories are on the same filesystem (have
     the same device identifier).
     '''
     return {
-        directory: os.stat(directory).st_dev if os.path.exists(directory) else None
+        directory: os.stat(full_directory).st_dev if os.path.exists(full_directory) else None
         for directory in directories
+        for full_directory in (os.path.join(working_directory or '', directory),)
     }
 
 
@@ -215,18 +221,13 @@ def make_list_filter_flags(local_borg_version, dry_run):
         return f'{base_flags}-'
 
 
-def collect_borgmatic_source_directories(borgmatic_source_directory):
+def collect_borgmatic_runtime_directories(borgmatic_runtime_directory):
     '''
-    Return a list of borgmatic-specific source directories used for state like database backups.
+    Return a list of borgmatic-specific runtime directories used for temporary runtime data like
+    streaming database dumps and bootstrap metadata. If no such directories exist, return an empty
+    list.
     '''
-    if not borgmatic_source_directory:
-        borgmatic_source_directory = state.DEFAULT_BORGMATIC_SOURCE_DIRECTORY
-
-    return (
-        [borgmatic_source_directory]
-        if os.path.exists(os.path.expanduser(borgmatic_source_directory))
-        else []
-    )
+    return [borgmatic_runtime_directory] if os.path.exists(borgmatic_runtime_directory) else []
 
 
 ROOT_PATTERN_PREFIX = 'R '
@@ -317,12 +318,8 @@ def check_all_source_directories_exist(source_directories, working_directory=Non
         for source_directory in source_directories
         if not all(
             [
-                os.path.exists(directory)
-                for directory in expand_directory(
-                    os.path.join(working_directory, source_directory)
-                    if working_directory
-                    else source_directory
-                )
+                os.path.exists(os.path.join(working_directory or '', directory))
+                for directory in expand_directory(source_directory, working_directory)
             ]
         )
     ]
@@ -330,6 +327,9 @@ def check_all_source_directories_exist(source_directories, working_directory=Non
         raise ValueError(f"Source directories do not exist: {', '.join(missing_directories)}")
 
 
+MAX_SPECIAL_FILE_PATHS_LENGTH = 1000
+
+
 def make_base_create_command(
     dry_run,
     repository_path,
@@ -337,7 +337,7 @@ def make_base_create_command(
     config_paths,
     local_borg_version,
     global_arguments,
-    borgmatic_source_directories,
+    borgmatic_runtime_directories,
     local_path='borg',
     remote_path=None,
     progress=False,
@@ -352,10 +352,7 @@ def make_base_create_command(
     (base Borg create command flags, Borg create command positional arguments, open pattern file
     handle, open exclude file handle).
     '''
-    try:
-        working_directory = os.path.expanduser(config.get('working_directory'))
-    except TypeError:
-        working_directory = None
+    working_directory = borgmatic.config.paths.get_working_directory(config)
 
     if config.get('source_directories_must_exist', False):
         check_all_source_directories_exist(
@@ -366,12 +363,16 @@ def make_base_create_command(
         map_directories_to_devices(
             expand_directories(
                 tuple(config.get('source_directories', ()))
-                + borgmatic_source_directories
-                + tuple(config_paths if config.get('store_config_files', True) else ())
+                + borgmatic_runtime_directories
+                + tuple(config_paths if config.get('store_config_files', True) else ()),
+                working_directory=working_directory,
             )
         ),
         additional_directory_devices=map_directories_to_devices(
-            expand_directories(pattern_root_directories(config.get('patterns')))
+            expand_directories(
+                pattern_root_directories(config.get('patterns')),
+                working_directory=working_directory,
+            )
         ),
     )
 
@@ -393,7 +394,9 @@ def make_base_create_command(
     lock_wait = config.get('lock_wait', None)
     list_filter_flags = make_list_filter_flags(local_borg_version, dry_run)
     files_cache = config.get('files_cache')
-    archive_name_format = config.get('archive_name_format', flags.DEFAULT_ARCHIVE_NAME_FORMAT)
+    archive_name_format = config.get(
+        'archive_name_format', flags.get_default_archive_name_format(local_borg_version)
+    )
     extra_borg_options = config.get('extra_borg_options', {}).get('create', '')
 
     if feature.available(feature.Feature.ATIME, local_borg_version):
@@ -431,7 +434,7 @@ def make_base_create_command(
         + (('--compression', compression) if compression else ())
         + upload_ratelimit_flags
         + (('--upload-buffer', str(upload_buffer_size)) if upload_buffer_size else ())
-        + (('--one-file-system',) if config.get('one_file_system') or stream_processes else ())
+        + (('--one-file-system',) if config.get('one_file_system') else ())
         + numeric_ids_flags
         + atime_flags
         + (('--noctime',) if config.get('ctime') is False else ())
@@ -471,12 +474,17 @@ def make_base_create_command(
             local_path,
             working_directory,
             borg_environment,
-            skip_directories=borgmatic_source_directories,
+            skip_directories=borgmatic_runtime_directories,
         )
 
         if special_file_paths:
+            truncated_special_file_paths = textwrap.shorten(
+                ', '.join(special_file_paths),
+                width=MAX_SPECIAL_FILE_PATHS_LENGTH,
+                placeholder=' ...',
+            )
             logger.warning(
-                f'{repository_path}: Excluding special files to prevent Borg from hanging: {", ".join(special_file_paths)}'
+                f'{repository_path}: Excluding special files to prevent Borg from hanging: {truncated_special_file_paths}'
             )
             exclude_file = write_pattern_file(
                 expand_home_directories(
@@ -513,8 +521,13 @@ def create_archive(
     create command while also triggering the given processes to produce output.
     '''
     borgmatic.logger.add_custom_log_levels()
-    borgmatic_source_directories = expand_directories(
-        collect_borgmatic_source_directories(config.get('borgmatic_source_directory'))
+
+    working_directory = borgmatic.config.paths.get_working_directory(config)
+    borgmatic_runtime_directories = expand_directories(
+        collect_borgmatic_runtime_directories(
+            borgmatic.config.paths.get_borgmatic_runtime_directory(config)
+        ),
+        working_directory=working_directory,
     )
 
     (create_flags, create_positional_arguments, pattern_file, exclude_file) = (
@@ -525,7 +538,7 @@ def create_archive(
             config_paths,
             local_borg_version,
             global_arguments,
-            borgmatic_source_directories,
+            borgmatic_runtime_directories,
             local_path,
             remote_path,
             progress,
@@ -546,11 +559,6 @@ def create_archive(
     # the terminal directly.
     output_file = DO_NOT_CAPTURE if progress else None
 
-    try:
-        working_directory = os.path.expanduser(config.get('working_directory'))
-    except TypeError:
-        working_directory = None
-
     borg_environment = environment.make_environment(config)
 
     create_flags += (
