@@ -6,7 +6,7 @@ from argparse import ArgumentParser
 from borgmatic.config import collect
 
 ACTION_ALIASES = {
-    'rcreate': ['init', '-I'],
+    'repo-create': ['rcreate', 'init', '-I'],
     'prune': ['-p'],
     'compact': [],
     'create': ['-C'],
@@ -18,10 +18,10 @@ ACTION_ALIASES = {
     'mount': ['-m'],
     'umount': ['-u'],
     'restore': ['-r'],
-    'rdelete': [],
-    'rlist': [],
+    'repo-delete': ['rdelete'],
+    'repo-list': ['rlist'],
     'list': ['-l'],
-    'rinfo': [],
+    'repo-info': ['rinfo'],
     'info': ['-i'],
     'transfer': [],
     'break-lock': [],
@@ -74,11 +74,11 @@ def omit_values_colliding_with_action_names(unparsed_arguments, parsed_arguments
     for action_name, parsed in parsed_arguments.items():
         for value in vars(parsed).values():
             if isinstance(value, str):
-                if value in ACTION_ALIASES.keys():
+                if value in ACTION_ALIASES.keys() and value in remaining_arguments:
                     remaining_arguments.remove(value)
             elif isinstance(value, list):
                 for item in value:
-                    if item in ACTION_ALIASES.keys():
+                    if item in ACTION_ALIASES.keys() and item in remaining_arguments:
                         remaining_arguments.remove(item)
 
     return tuple(remaining_arguments)
@@ -300,7 +300,7 @@ def make_parsers():
         '--config',
         dest='config_paths',
         action='append',
-        help=f"Configuration filename or directory, can specify flag multiple times, defaults to: {' '.join(unexpanded_config_paths)}",
+        help=f"Configuration filename or directory, can specify flag multiple times, defaults to: -c {' -c '.join(unexpanded_config_paths)}",
     )
     global_group.add_argument(
         '-n',
@@ -402,51 +402,51 @@ def make_parsers():
         metavar='',
         help='Specify zero or more actions. Defaults to create, prune, compact, and check. Use --help with action for details:',
     )
-    rcreate_parser = action_parsers.add_parser(
-        'rcreate',
-        aliases=ACTION_ALIASES['rcreate'],
+    repo_create_parser = action_parsers.add_parser(
+        'repo-create',
+        aliases=ACTION_ALIASES['repo-create'],
         help='Create a new, empty Borg repository',
         description='Create a new, empty Borg repository',
         add_help=False,
     )
-    rcreate_group = rcreate_parser.add_argument_group('rcreate arguments')
-    rcreate_group.add_argument(
+    repo_create_group = repo_create_parser.add_argument_group('repo-create arguments')
+    repo_create_group.add_argument(
         '-e',
         '--encryption',
         dest='encryption_mode',
         help='Borg repository encryption mode',
         required=True,
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '--source-repository',
         '--other-repo',
         metavar='KEY_REPOSITORY',
         help='Path to an existing Borg repository whose key material should be reused [Borg 2.x+ only]',
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '--repository',
-        help='Path of the new repository to create (must be already specified in a borgmatic configuration file), defaults to the configured repository if there is only one',
+        help='Path of the new repository to create (must be already specified in a borgmatic configuration file), defaults to the configured repository if there is only one, quoted globs supported',
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '--copy-crypt-key',
         action='store_true',
         help='Copy the crypt key used for authenticated encryption from the source repository, defaults to a new random key [Borg 2.x+ only]',
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '--append-only',
         action='store_true',
         help='Create an append-only repository',
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '--storage-quota',
         help='Create a repository with a fixed storage quota',
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '--make-parent-dirs',
         action='store_true',
         help='Create any missing parent directories of the repository directory',
     )
-    rcreate_group.add_argument(
+    repo_create_group.add_argument(
         '-h', '--help', action='help', help='Show this help message and exit'
     )
 
@@ -460,7 +460,7 @@ def make_parsers():
     transfer_group = transfer_parser.add_argument_group('transfer arguments')
     transfer_group.add_argument(
         '--repository',
-        help='Path of existing destination repository to transfer archives to, defaults to the configured repository if there is only one',
+        help='Path of existing destination repository to transfer archives to, defaults to the configured repository if there is only one, quoted globs supported',
     )
     transfer_group.add_argument(
         '--source-repository',
@@ -469,7 +469,7 @@ def make_parsers():
     )
     transfer_group.add_argument(
         '--archive',
-        help='Name of single archive to transfer (or "latest"), defaults to transferring all archives',
+        help='Name or hash of a single archive to transfer (or "latest"), defaults to transferring all archives',
     )
     transfer_group.add_argument(
         '--upgrader',
@@ -486,7 +486,7 @@ def make_parsers():
         '--match-archives',
         '--glob-archives',
         metavar='PATTERN',
-        help='Only transfer archives with names matching this pattern',
+        help='Only transfer archives with names, hashes, or series matching this pattern',
     )
     transfer_group.add_argument(
         '--sort-by', metavar='KEYS', help='Comma-separated list of sorting keys'
@@ -533,7 +533,14 @@ def make_parsers():
     prune_group = prune_parser.add_argument_group('prune arguments')
     prune_group.add_argument(
         '--repository',
-        help='Path of specific existing repository to prune (must be already specified in a borgmatic configuration file)',
+        help='Path of specific existing repository to prune (must be already specified in a borgmatic configuration file), quoted globs supported',
+    )
+    prune_group.add_argument(
+        '-a',
+        '--match-archives',
+        '--glob-archives',
+        metavar='PATTERN',
+        help='When pruning, only consider archives with names, hashes, or series matching this pattern',
     )
     prune_group.add_argument(
         '--stats',
@@ -577,7 +584,7 @@ def make_parsers():
     compact_group = compact_parser.add_argument_group('compact arguments')
     compact_group.add_argument(
         '--repository',
-        help='Path of specific existing repository to compact (must be already specified in a borgmatic configuration file)',
+        help='Path of specific existing repository to compact (must be already specified in a borgmatic configuration file), quoted globs supported',
     )
     compact_group.add_argument(
         '--progress',
@@ -613,7 +620,7 @@ def make_parsers():
     create_group = create_parser.add_argument_group('create arguments')
     create_group.add_argument(
         '--repository',
-        help='Path of specific existing repository to backup to (must be already specified in a borgmatic configuration file)',
+        help='Path of specific existing repository to backup to (must be already specified in a borgmatic configuration file), quoted globs supported',
     )
     create_group.add_argument(
         '--progress',
@@ -647,7 +654,7 @@ def make_parsers():
     check_group = check_parser.add_argument_group('check arguments')
     check_group.add_argument(
         '--repository',
-        help='Path of specific existing repository to check (must be already specified in a borgmatic configuration file)',
+        help='Path of specific existing repository to check (must be already specified in a borgmatic configuration file), quoted globs supported',
     )
     check_group.add_argument(
         '--progress',
@@ -673,7 +680,7 @@ def make_parsers():
         '--match-archives',
         '--glob-archives',
         metavar='PATTERN',
-        help='Only check archives with names matching this pattern',
+        help='Only check archives with names, hashes, or series matching this pattern',
     )
     check_group.add_argument(
         '--only',
@@ -701,11 +708,11 @@ def make_parsers():
     delete_group = delete_parser.add_argument_group('delete arguments')
     delete_group.add_argument(
         '--repository',
-        help='Path of repository to delete or delete archives from, defaults to the configured repository if there is only one',
+        help='Path of repository to delete or delete archives from, defaults to the configured repository if there is only one, quoted globs supported',
     )
     delete_group.add_argument(
         '--archive',
-        help='Archive to delete',
+        help='Archive name, hash, or series to delete',
     )
     delete_group.add_argument(
         '--list',
@@ -749,7 +756,7 @@ def make_parsers():
         '--match-archives',
         '--glob-archives',
         metavar='PATTERN',
-        help='Only delete archives matching this pattern',
+        help='Only delete archives with names, hashes, or series matching this pattern',
     )
     delete_group.add_argument(
         '--sort-by', metavar='KEYS', help='Comma-separated list of sorting keys'
@@ -792,10 +799,10 @@ def make_parsers():
     extract_group = extract_parser.add_argument_group('extract arguments')
     extract_group.add_argument(
         '--repository',
-        help='Path of repository to extract, defaults to the configured repository if there is only one',
+        help='Path of repository to extract, defaults to the configured repository if there is only one, quoted globs supported',
     )
     extract_group.add_argument(
-        '--archive', help='Name of archive to extract (or "latest")', required=True
+        '--archive', help='Name or hash of a single archive to extract (or "latest")', required=True
     )
     extract_group.add_argument(
         '--path',
@@ -854,16 +861,30 @@ def make_parsers():
     )
     config_bootstrap_group.add_argument(
         '--repository',
-        help='Path of repository to extract config files from',
+        help='Path of repository to extract config files from, quoted globs supported',
         required=True,
     )
+    config_bootstrap_group.add_argument(
+        '--local-path',
+        help='Alternate Borg local executable. Defaults to "borg"',
+        default='borg',
+    )
+    config_bootstrap_group.add_argument(
+        '--remote-path',
+        help='Alternate Borg remote executable. Defaults to "borg"',
+        default='borg',
+    )
+    config_bootstrap_group.add_argument(
+        '--user-runtime-directory',
+        help='Path used for temporary runtime data like bootstrap metadata. Defaults to $XDG_RUNTIME_DIR or /var/run/$UID',
+    )
     config_bootstrap_group.add_argument(
         '--borgmatic-source-directory',
-        help='Path that stores the config files used to create an archive and additional source files used for temporary internal state like borgmatic database dumps. Defaults to ~/.borgmatic',
+        help='Deprecated. Path formerly used for temporary runtime data like bootstrap metadata. Defaults to ~/.borgmatic',
     )
     config_bootstrap_group.add_argument(
         '--archive',
-        help='Name of archive to extract config files from, defaults to "latest"',
+        help='Name or hash of a single archive to extract config files from, defaults to "latest"',
         default='latest',
     )
     config_bootstrap_group.add_argument(
@@ -952,10 +973,10 @@ def make_parsers():
     export_tar_group = export_tar_parser.add_argument_group('export-tar arguments')
     export_tar_group.add_argument(
         '--repository',
-        help='Path of repository to export from, defaults to the configured repository if there is only one',
+        help='Path of repository to export from, defaults to the configured repository if there is only one, quoted globs supported',
     )
     export_tar_group.add_argument(
-        '--archive', help='Name of archive to export (or "latest")', required=True
+        '--archive', help='Name or hash of a single archive to export (or "latest")', required=True
     )
     export_tar_group.add_argument(
         '--path',
@@ -998,9 +1019,11 @@ def make_parsers():
     mount_group = mount_parser.add_argument_group('mount arguments')
     mount_group.add_argument(
         '--repository',
-        help='Path of repository to use, defaults to the configured repository if there is only one',
+        help='Path of repository to use, defaults to the configured repository if there is only one, quoted globs supported',
+    )
+    mount_group.add_argument(
+        '--archive', help='Name or hash of a single archive to mount (or "latest")'
     )
-    mount_group.add_argument('--archive', help='Name of archive to mount (or "latest")')
     mount_group.add_argument(
         '--mount-point',
         metavar='PATH',
@@ -1070,40 +1093,40 @@ def make_parsers():
     )
     umount_group.add_argument('-h', '--help', action='help', help='Show this help message and exit')
 
-    rdelete_parser = action_parsers.add_parser(
-        'rdelete',
-        aliases=ACTION_ALIASES['rdelete'],
+    repo_delete_parser = action_parsers.add_parser(
+        'repo-delete',
+        aliases=ACTION_ALIASES['repo-delete'],
         help='Delete an entire repository (with Borg 1.2+, you must run compact afterwards to actually free space)',
         description='Delete an entire repository (with Borg 1.2+, you must run compact afterwards to actually free space)',
         add_help=False,
     )
-    rdelete_group = rdelete_parser.add_argument_group('delete arguments')
-    rdelete_group.add_argument(
+    repo_delete_group = repo_delete_parser.add_argument_group('delete arguments')
+    repo_delete_group.add_argument(
         '--repository',
-        help='Path of repository to delete, defaults to the configured repository if there is only one',
+        help='Path of repository to delete, defaults to the configured repository if there is only one, quoted globs supported',
     )
-    rdelete_group.add_argument(
+    repo_delete_group.add_argument(
         '--list',
         dest='list_archives',
         action='store_true',
         help='Show details for the archives in the given repository',
     )
-    rdelete_group.add_argument(
+    repo_delete_group.add_argument(
         '--force',
         action='count',
         help='Force deletion of corrupted archives, can be given twice if once does not work',
     )
-    rdelete_group.add_argument(
+    repo_delete_group.add_argument(
         '--cache-only',
         action='store_true',
         help='Delete only the local cache for the given repository',
     )
-    rdelete_group.add_argument(
+    repo_delete_group.add_argument(
         '--keep-security-info',
         action='store_true',
         help='Do not delete the local security info when deleting a repository',
     )
-    rdelete_group.add_argument(
+    repo_delete_group.add_argument(
         '-h', '--help', action='help', help='Show this help message and exit'
     )
 
@@ -1117,10 +1140,12 @@ def make_parsers():
     restore_group = restore_parser.add_argument_group('restore arguments')
     restore_group.add_argument(
         '--repository',
-        help='Path of repository to restore from, defaults to the configured repository if there is only one',
+        help='Path of repository to restore from, defaults to the configured repository if there is only one, quoted globs supported',
     )
     restore_group.add_argument(
-        '--archive', help='Name of archive to restore from (or "latest")', required=True
+        '--archive',
+        help='Name or hash of a single archive to restore from (or "latest")',
+        required=True,
     )
     restore_group.add_argument(
         '--data-source',
@@ -1161,65 +1186,67 @@ def make_parsers():
         '-h', '--help', action='help', help='Show this help message and exit'
     )
 
-    rlist_parser = action_parsers.add_parser(
-        'rlist',
-        aliases=ACTION_ALIASES['rlist'],
+    repo_list_parser = action_parsers.add_parser(
+        'repo-list',
+        aliases=ACTION_ALIASES['repo-list'],
         help='List repository',
         description='List the archives in a repository',
         add_help=False,
     )
-    rlist_group = rlist_parser.add_argument_group('rlist arguments')
-    rlist_group.add_argument(
+    repo_list_group = repo_list_parser.add_argument_group('repo-list arguments')
+    repo_list_group.add_argument(
         '--repository',
-        help='Path of repository to list, defaults to the configured repositories',
+        help='Path of repository to list, defaults to the configured repositories, quoted globs supported',
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--short', default=False, action='store_true', help='Output only archive names'
     )
-    rlist_group.add_argument('--format', help='Format for archive listing')
-    rlist_group.add_argument(
+    repo_list_group.add_argument('--format', help='Format for archive listing')
+    repo_list_group.add_argument(
         '--json', default=False, action='store_true', help='Output results as JSON'
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '-P', '--prefix', help='Deprecated. Only list archive names starting with this prefix'
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '-a',
         '--match-archives',
         '--glob-archives',
         metavar='PATTERN',
-        help='Only list archive names matching this pattern',
+        help='Only list archive names, hashes, or series matching this pattern',
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--sort-by', metavar='KEYS', help='Comma-separated list of sorting keys'
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--first', metavar='N', help='List first N archives after other filters are applied'
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--last', metavar='N', help='List last N archives after other filters are applied'
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--oldest',
         metavar='TIMESPAN',
         help='List archives within a specified time range starting from the timestamp of the oldest archive (e.g. 7d or 12m) [Borg 2.x+ only]',
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--newest',
         metavar='TIMESPAN',
         help='List archives within a time range that ends at timestamp of the newest archive and starts a specified time range ago (e.g. 7d or 12m) [Borg 2.x+ only]',
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--older',
         metavar='TIMESPAN',
         help='List archives that are older than the specified time range (e.g. 7d or 12m) from the current time [Borg 2.x+ only]',
     )
-    rlist_group.add_argument(
+    repo_list_group.add_argument(
         '--newer',
         metavar='TIMESPAN',
         help='List archives that are newer than the specified time range (e.g. 7d or 12m) from the current time [Borg 2.x+ only]',
     )
-    rlist_group.add_argument('-h', '--help', action='help', help='Show this help message and exit')
+    repo_list_group.add_argument(
+        '-h', '--help', action='help', help='Show this help message and exit'
+    )
 
     list_parser = action_parsers.add_parser(
         'list',
@@ -1231,9 +1258,11 @@ def make_parsers():
     list_group = list_parser.add_argument_group('list arguments')
     list_group.add_argument(
         '--repository',
-        help='Path of repository containing archive to list, defaults to the configured repositories',
+        help='Path of repository containing archive to list, defaults to the configured repositories, quoted globs supported',
+    )
+    list_group.add_argument(
+        '--archive', help='Name or hash of a single archive to list (or "latest")'
     )
-    list_group.add_argument('--archive', help='Name of the archive to list (or "latest")')
     list_group.add_argument(
         '--path',
         metavar='PATH',
@@ -1288,22 +1317,24 @@ def make_parsers():
     )
     list_group.add_argument('-h', '--help', action='help', help='Show this help message and exit')
 
-    rinfo_parser = action_parsers.add_parser(
-        'rinfo',
-        aliases=ACTION_ALIASES['rinfo'],
+    repo_info_parser = action_parsers.add_parser(
+        'repo-info',
+        aliases=ACTION_ALIASES['repo-info'],
         help='Show repository summary information such as disk space used',
         description='Show repository summary information such as disk space used',
         add_help=False,
     )
-    rinfo_group = rinfo_parser.add_argument_group('rinfo arguments')
-    rinfo_group.add_argument(
+    repo_info_group = repo_info_parser.add_argument_group('repo-info arguments')
+    repo_info_group.add_argument(
         '--repository',
-        help='Path of repository to show info for, defaults to the configured repository if there is only one',
+        help='Path of repository to show info for, defaults to the configured repository if there is only one, quoted globs supported',
     )
-    rinfo_group.add_argument(
+    repo_info_group.add_argument(
         '--json', dest='json', default=False, action='store_true', help='Output results as JSON'
     )
-    rinfo_group.add_argument('-h', '--help', action='help', help='Show this help message and exit')
+    repo_info_group.add_argument(
+        '-h', '--help', action='help', help='Show this help message and exit'
+    )
 
     info_parser = action_parsers.add_parser(
         'info',
@@ -1315,9 +1346,11 @@ def make_parsers():
     info_group = info_parser.add_argument_group('info arguments')
     info_group.add_argument(
         '--repository',
-        help='Path of repository containing archive to show info for, defaults to the configured repository if there is only one',
+        help='Path of repository containing archive to show info for, defaults to the configured repository if there is only one, quoted globs supported',
+    )
+    info_group.add_argument(
+        '--archive', help='Archive name, hash, or series to show info for (or "latest")'
     )
-    info_group.add_argument('--archive', help='Name of archive to show info for (or "latest")')
     info_group.add_argument(
         '--json', dest='json', default=False, action='store_true', help='Output results as JSON'
     )
@@ -1331,7 +1364,7 @@ def make_parsers():
         '--match-archives',
         '--glob-archives',
         metavar='PATTERN',
-        help='Only show info for archive names matching this pattern',
+        help='Only show info for archive names, hashes, or series matching this pattern',
     )
     info_group.add_argument(
         '--sort-by', metavar='KEYS', help='Comma-separated list of sorting keys'
@@ -1376,7 +1409,7 @@ def make_parsers():
     break_lock_group = break_lock_parser.add_argument_group('break-lock arguments')
     break_lock_group.add_argument(
         '--repository',
-        help='Path of repository to break the lock for, defaults to the configured repository if there is only one',
+        help='Path of repository to break the lock for, defaults to the configured repository if there is only one, quoted globs supported',
     )
     break_lock_group.add_argument(
         '-h', '--help', action='help', help='Show this help message and exit'
@@ -1416,7 +1449,7 @@ def make_parsers():
     )
     key_export_group.add_argument(
         '--repository',
-        help='Path of repository to export the key for, defaults to the configured repository if there is only one',
+        help='Path of repository to export the key for, defaults to the configured repository if there is only one, quoted globs supported',
     )
     key_export_group.add_argument(
         '--path',
@@ -1427,6 +1460,23 @@ def make_parsers():
         '-h', '--help', action='help', help='Show this help message and exit'
     )
 
+    key_change_passphrase_parser = key_parsers.add_parser(
+        'change-passphrase',
+        help='Change the passphrase protecting the repository key',
+        description='Change the passphrase protecting the repository key',
+        add_help=False,
+    )
+    key_change_passphrase_group = key_change_passphrase_parser.add_argument_group(
+        'key change-passphrase arguments'
+    )
+    key_change_passphrase_group.add_argument(
+        '--repository',
+        help='Path of repository to change the passphrase for, defaults to the configured repository if there is only one, quoted globs supported',
+    )
+    key_change_passphrase_group.add_argument(
+        '-h', '--help', action='help', help='Show this help message and exit'
+    )
+
     borg_parser = action_parsers.add_parser(
         'borg',
         aliases=ACTION_ALIASES['borg'],
@@ -1437,9 +1487,11 @@ def make_parsers():
     borg_group = borg_parser.add_argument_group('borg arguments')
     borg_group.add_argument(
         '--repository',
-        help='Path of repository to pass to Borg, defaults to the configured repositories',
+        help='Path of repository to pass to Borg, defaults to the configured repositories, quoted globs supported',
+    )
+    borg_group.add_argument(
+        '--archive', help='Archive name, hash, or series to pass to Borg (or "latest")'
     )
-    borg_group.add_argument('--archive', help='Name of archive to pass to Borg (or "latest")')
     borg_group.add_argument(
         '--',
         metavar='OPTION',
@@ -1498,9 +1550,9 @@ def parse_arguments(*unparsed_arguments):
         )
 
     if (
-        ('list' in arguments and 'rinfo' in arguments and arguments['list'].json)
+        ('list' in arguments and 'repo-info' in arguments and arguments['list'].json)
         or ('list' in arguments and 'info' in arguments and arguments['list'].json)
-        or ('rinfo' in arguments and 'info' in arguments and arguments['rinfo'].json)
+        or ('repo-info' in arguments and 'info' in arguments and arguments['repo-info'].json)
     ):
         raise ValueError('With the --json flag, multiple actions cannot be used together.')
 
@@ -1518,9 +1570,11 @@ def parse_arguments(*unparsed_arguments):
             'With the list action, only one of --prefix or --match-archives flags can be used.'
         )
 
-    if 'rlist' in arguments and (arguments['rlist'].prefix and arguments['rlist'].match_archives):
+    if 'repo-list' in arguments and (
+        arguments['repo-list'].prefix and arguments['repo-list'].match_archives
+    ):
         raise ValueError(
-            'With the rlist action, only one of --prefix or --match-archives flags can be used.'
+            'With the repo-list action, only one of --prefix or --match-archives flags can be used.'
         )
 
     if 'info' in arguments and (
